--[[
	WoWLogs

	Project started: 2018-10-04
	Approx hours spent: 150
	Date:    2018-10-19
	Author:  Petoovee/Petomatick
]]--


if(db == nil) then db = {} end
local hooked = {} -- Hooked elements
local text = {} -- Unknown use
local syncKeys = {} -- db keys in readily accessible form
local WoWLogsTicker = true -- Enable periodic syncing?

local WoWLogsTooltip = CreateFrame( "Frame", "WoWLogsTooltip", UIParent);
WoWLogsTooltip:SetPoint("CENTER", UIParent, "CENTER")
if(db["WoWLogs_xOfs"] ~= nil and db["WoWLogs_yOfs"] ~= nil) then
	C_Timer.After(1, function()
		WoWLogsTooltip:SetPoint("CENTER", UIParent, "CENTER", db["WoWLogs_xOfs"], db["WoWLogs_yOfs"])
	end)
end
WoWLogsTooltip:SetMovable(true)
WoWLogsTooltip:EnableMouse(true)
WoWLogsTooltip:RegisterForDrag("LeftButton")
WoWLogsTooltip:SetScript("OnDragStart", WoWLogsTooltip.StartMoving)
WoWLogsTooltip:SetScript("OnDragStop", WoWLogsTooltip.StopMovingOrSizing)
WoWLogsTooltip:HookScript("OnDragStop", function() 
	db["WoWLogs_xOfs"] = select(4, WoWLogsTooltip:GetPoint())
	db["WoWLogs_yOfs"] = select(5, WoWLogsTooltip:GetPoint())
	WoWLogsSilentPrint("x: " ..db["WoWLogs_xOfs"] .." | y: " ..db["WoWLogs_yOfs"])
end)
WoWLogsTooltip.texture = WoWLogsTooltip:CreateTexture("WoWLogsUITexture", "ARTWORK")
WoWLogsTooltip.texture:SetPoint("CENTER", WoWLogsTooltip, "CENTER", 0, 0)
WoWLogsTooltip.texture:SetColorTexture(1, 1, 1, 0.1)
WoWLogsTooltip.centerText = WoWLogsTooltip:CreateFontString(nil, "OVERLAY")
WoWLogsTooltip.centerText:SetFontObject("GameFontNormal")
WoWLogsTooltip.centerText:SetPoint("CENTER", WoWLogsTooltip, "CENTER", 0, 0)
WoWLogsTooltip.centerText:SetText("Welcome back")
WoWLogsTooltip:SetSize(WoWLogsTooltip.centerText:GetWidth(), WoWLogsTooltip.centerText:GetHeight())
WoWLogsTooltip.texture:SetSize(WoWLogsTooltip.centerText:GetWidth(), WoWLogsTooltip.centerText:GetHeight())
WoWLogsTooltip:Hide()
if(db["WoWLogs_Debug"] == nil) then
	db["WoWLogs_Debug"] = true
end

-- Purge data from the future. not sure when this will be worthwhile.
-- Alternative would be for every string to pass a test when entering our database through p2p
-- Although the Updater is not flawless either... Maybe purge when sending it aswell?
function purgeFutureData()
	for x,y in pairs(db) do
		if(db[x]>date("%y%m%d%H")) then
			db[x] = nil
		end
	end
end


-- ###### P2P Module START ######
local WoWLogsEventFrame,events = CreateFrame("FRAME"),{};
function events:CHAT_MSG_ADDON(prefix, text, type, sender)
	if(prefix == "WoWLogs" and sender ~= UnitName("player") .."-" ..string.gsub(GetRealmName()," ", "")) then
		WoWLogsSilentPrint("Recieved package from " ..sender)
		local textString = text ..""
		for name in string.gmatch(textString, "([^:]*):") do -- Stole this from the web, no idea how it works
			if(strmatch(name, "0") == nil and string.len(name) > 2) then
				WoWLogsNewEntry(name)
			end
		end
	end
end
C_ChatInfo.RegisterAddonMessagePrefix("WoWLogs")
-- ###### P2P Module END ######

-- ###### SHOW MOUSEOVER SCORES START ######
function events:UPDATE_MOUSEOVER_UNIT()
	if(UnitIsPlayer("mouseover") and UnitLevel("mouseover") == 120 and UnitAffectingCombat("player") == false ) then
		local name, realm = UnitName("mouseover")
		if(realm == nil) then
			name = name.."-" ..string.gsub(GetRealmName()," ", "")
		else
			name = name .."-" ..realm
		end
		WoWLogsShowScores(name)
		if(db["WoWLogs_UIState"] == "mouseover") then
			WoWLogsTooltip:Show()
			WoWLogsMouseoverTicker = C_Timer.NewTicker(0.1, function()
				if(not UnitIsPlayer("mouseover") or not UnitLevel("mouseover") == 120 ) then
					WoWLogsTooltip:Hide()
					WoWLogsMouseoverTicker:Cancel()
				end
			end, 600)
		end
	end
end
-- ###### SHOW MOUSEOVER SCORES END ######

-- ###### WHO LIST INTEGRATION START ######
local zoneTable = {"Orgrimmar", "Nazmir", "Zuldazar", "Vol'dun", "Drustvar", "Stormsong Valley", "Tiragarde Sound", "Dalaran"}
local function WoWLogsWhoScan()
	FriendsFrame:UnregisterEvent("WHO_LIST_UPDATE")
	SendWho("z-" ..zoneTable[math.random(#zoneTable)] .." 120")
	C_Timer.After(1, function()
		FriendsFrame:RegisterEvent("WHO_LIST_UPDATE")
	end)
	for i = 1, GetNumWhoResults() do
		local name, _, level = GetWhoInfo(i)
		if(level == 120) then
			if(strmatch(name, "-") == nil) then
				name = name.."-" ..string.gsub(GetRealmName()," ", "")
			end
			WoWLogsNewEntry(name)
		end
	end
end
-- ###### WHO LIST INTEGRATION END ######

-- ###### FUNCTIONS START ######
-- For easier management, even if we call it on so few placed
function WoWLogsNewEntry(name)
	if(not IsAddOnLoaded("WoWLogsDB")) then
		return;
	end
	if(strmatch(name, "0") ~= nil) then
		return;
	end
	if(staticdb[name] == nil and db[name] == nil) then
		db[name] = "00000000000000000000000000000000000"
		table.insert(syncKeys, name)
		WoWLogsSilentPrint(name .." added to fetch list!")
	end
end

function WoWLogsSilentPrint(string)
	if(db["WoWLogs_Debug"] == false) then
		print(string)
	end
end

-- Parses our delicate strings for our frame
function WoWLogsShowScores(name)
	if(not IsAddOnLoaded("WoWLogsDB")) then
		return;
	end
	local date, mythicPlus, mythicDPS, mythicHPS, heroicDPS, heroicHPS, normalDPS, normalHPS, lfrDPS, lfrHPS;
	local text = ""
	local dbString = ""
	local months = {["01"] = " Jan ",
	["02"] = " Feb ",["03"] = " Mar ",
	["04"] = " Apr ",["05"] = " May ",
	["06"] = " Jun ",["07"] = " Jul ",
	["08"] = " Aug ",["09"] = " Sep ",
	["10"] = " Oct ",["11"] = " Nov ",
	["12"] = " Dec "}
	if(staticdb[name] == nil) then
		WoWLogsNewEntry(name)
		dbString = db[name]
	else
		dbString = staticdb[name]
	end
	if(string.sub(dbString,1,8) == "00000000") then
		date = "Never"
	else
		date = string.sub(dbString,3,4) ..months[string.sub(dbString,1,2)] ..string.sub(dbString,5,6) ..":" ..string.sub(dbString,7,8)
	end
	mythicPlus = string.sub(dbString,9,11)
	mythicDPS = string.sub(dbString,12,14)
	mythicHPS = string.sub(dbString,15,17)
	heroicDPS = string.sub(dbString,18,20)
	heroicHPS = string.sub(dbString,21,23)
	normalDPS = string.sub(dbString,24,26)
	normalHPS = string.sub(dbString,27,29)
	lfrDPS = string.sub(dbString,30,32)
	lfrHPS = string.sub(dbString,33,37)
	if(string.find(name, "-") > 10) then
		text = string.sub(name, 1, string.find(name, "-")-1) .."\n" ..string.sub(name, string.find(name, "-")+1, string.len(name))
	else
		text = name
	end
	if(date == "Never") then
		text = text .."\n" .."Never updated"
	else
		text = text .."\n" ..date
	end
	if(mythicPlus ~= nil) then
		text = text .."\n\n" .."Mythic+: " ..mythicPlus
	end
	if(mythicDPS ~= nil) then
		text = text .."\n\n" .."Mythic DPS: " ..mythicDPS
	end	
	if(mythicHPS ~= nil) then
		text = text .."\n" .."Mythic HPS: " ..mythicHPS
	end
	if(heroicDPS ~= nil) then
		text = text .."\n" .."Heroic DPS: " ..heroicDPS
	end	
	if(heroicHPS ~= nil) then
		text = text .."\n" .."Heroic HPS: " ..heroicHPS
	end
	if(normalDPS ~= nil) then
		text = text .."\n" .."Normal DPS: " ..normalDPS
	end
	if(normalHPS ~= nil) then
		text = text .."\n" .."Normal HPS: " ..normalHPS
	end
	-- if(lfrDPS ~= nil) then
	-- 	text = text .."\n" .."LFR DPS: " ..lfrDPS
	-- end
	-- if(lfrHPS ~= nil) then
	-- 	text = text .."\n" .."LFR HPS: " ..lfrHPS
	-- end
	WoWLogsTooltip.centerText:SetText(text)
	WoWLogsTooltip:SetSize(WoWLogsTooltip.centerText:GetWidth()+10, WoWLogsTooltip.centerText:GetHeight()+10)
	WoWLogsTooltip.texture:SetSize(WoWLogsTooltip.centerText:GetWidth()+10, WoWLogsTooltip.centerText:GetHeight()+10)
end

-- Seemingly best way for us to get hold of the numbers...
local function GetName(applicantID)
	for btnNbr=1,20 do
		local name = select(1, C_LFGList.GetApplicantMemberInfo(applicantID, btnNbr))
		if(name ~= nil) then
			return name;
		end
	end
	WoWLogsSilentPrint("btnNbr out of bounds")
end

-- ###### FUNCTIONS END ######
-- ###### HOOKS START ######

local function OnShowHook(self)
	name = GetName(self.applicantID)
	if(not string.find(name, "-")) then
		name = name.."-" ..string.gsub(GetRealmName()," ", "")
	end
	WoWLogsShowScores(name)
end

local function OnLeaveHook(self)
	name = UnitName("player") .."-" ..string.gsub(GetRealmName()," ", "")
	WoWLogsShowScores(name)
end

hooksecurefunc("LFGListApplicationViewer_UpdateResults", function(self)
	local buttons = self.ScrollFrame.buttons
	for i = 1, #buttons do
		local button = buttons[i]
		if not hooked[button] then
			button:HookScript("OnShow", OnShowHook)
			button:HookScript("OnLeave", OnLeaveHook)
			hooked[button] = true
		end
	end
end)

-- Show scores for the member at the mouse
local function OnEnterHookMember(self)
	local name, class, localizedClass, level, itemLevel, tank, healer, damage, assignedRole = C_LFGList.GetApplicantMemberInfo(self:GetParent().applicantID, self.memberIdx);
	WoWLogsShowScores(name)
end 

-- Hook above script, and show ourselves when we leave the member area
-- So what we know is that the hook can forward the variables used for the hooked function to our function
-- Which we can use for our own needs
hooksecurefunc("LFGListApplicationViewer_UpdateApplicant", function(button, id)
	local id, status, pendingStatus, numMembers, isNew = C_LFGList.GetApplicantInfo(id);
	--Update individual members
	for i=1, numMembers do
		local member = button.Members[i];
		if(member) then
			if not hooked[member] then
				member:HookScript("OnEnter", OnEnterHookMember)
				member:HookScript("OnLeave", OnLeaveHook)
				hooked[member] = true
			end
		end
	end
end)

-- ###### HOOKS END ######

-- Periodic random sync START
local syncTicker = 0
local dbSize = 0 -- How many entries do we have?
local loggedUsers = 0 -- How many do actually have logs??
local function tickerFunctions()

	-- Moved down here for delayed execution
	if(syncKeys[1] == nil and dbSize == 0) then		-- Count and store all the keys for syncing, and remove faulty entries
		LoadAddOn("WoWLogsDB")
		-- JoinTemporaryChannel("WoWLogs", nil, ChatFrame1:GetID()) -- Cheap attempt at global sync
		C_ChatInfo.RegisterAddonMessagePrefix("WoWLogs")
		for key in pairs(staticdb) do
			if(string.len(staticdb[key]) == 35) then
				if( db[key] ~= nil) then
					db[key] = nil
					WoWLogsSilentPrint(key .." was found in the shared database and was therefore removed.")
				end
			end
			if(string.sub(staticdb[key],9,35) ~= "000000000000000000000000000") then
				loggedUsers = loggedUsers + 1
			end
			dbSize = dbSize + 1
		end

		for key in pairs(db) do
			if(string.sub(key,1,7) ~= "WoWLogs") then 
				if(string.len(db[key]) ~= 35 or strmatch(key, "0") ~= nil or string.find(key, "-") == nil or string.find(key, "-", string.find(key, "-")+1) ~= nil) then
					WoWLogsSilentPrint("WoWLogs: Removing faulty entry for " ..key)
					db[key] = nil
				else
					if(string.sub(db[key],1,8) == "00000000") then
						table.insert(syncKeys, key)
					end
				end
			end
			-- if(staticdb[key] ~= nil) then
			-- 	db[key] = nil;
			-- WoWLogsSilentPrint(key .." was found in the shared database and was therefore removed.")
			-- end
		end
		print("WoWLogs db contains " ..dbSize .." users currently, of which " ..loggedUsers .." have logs.")
		WoWLogsSilentPrint(#syncKeys .." need to be synced to master.")
		-- Apparently this wasnt loaded in time!
		if(db["WoWLogs_UIState"] == nil) then
			db["WoWLogs_UIState"] = "alwayslfg"
		end
		if(db["WoWLogs_UIState"] == "never") then
			WoWLogsTooltip:Hide()
		end
		if(db["WoWLogs_UIState"] == "always") then
			WoWLogsTooltip:Show()
		end

		if(db["WoWLogs_WhoScanner"] == "yes") then
			C_Timer.NewTicker(15, WoWLogsWhoScan)
		end

	end


	-- -- Getting our fellow party members // Disabled since it appears to be intrusive and the entries/hr is much lower than the who-scanner
	-- if(syncTicker>4 and IsInGroup(LE_PARTY_CATEGORY_HOME) == true) then
	-- 	for i = 1, GetNumGroupMembers() do
	-- 		name = "" ..GetRaidRosterInfo(i) 
	-- 		if(not string.find(name, "-")) then
	-- 			name = name.."-" ..string.gsub(GetRealmName()," ", "")
	-- 		end

	-- 		WoWLogsNewEntry(name)
	-- 	end
	-- end

	-- The actual random syncing
	if(#syncKeys > 0)then
		local syncNames = ""
		for i = 1, 15 do
			local randKey = syncKeys[math.random(#syncKeys)]
			if(db[randKey] ~= true and db[randKey] ~= false and db[randKey] ~= nil) then
				if(string.find(randKey, "-", string.find(randKey, "-")+1) ~= nil) then
					WoWLogsSilentPrint("Found faulty key: " ..randKey)
				else
					if((string.len(syncNames) + string.len(randKey)) < 255) then
						syncNames = syncNames ..randKey ..":" 
					end
				end
			end
		end

		if(IsInGroup(LE_PARTY_CATEGORY_HOME) == true ) then
			C_ChatInfo.SendAddonMessage("WoWLogs", syncNames, "PARTY")
		end
		-- C_ChatInfo.SendAddonMessage("WoWLogs", randKey, "CHANNEL", "WoWLogs")
		if(syncTicker>4 and GetGuildInfo("player")~=nil) then -- For every 5 minutes send a package to the guild
				C_ChatInfo.SendAddonMessage("WoWLogs", syncNames, "GUILD")
		end
	end
	
	if(syncTicker>4) then
		syncTicker = 0
	else
		syncTicker = syncTicker + 1
	end
end
if(WoWLogsTicker == true) then
	C_Timer.NewTicker(60, tickerFunctions) -- Does quite a lot of things now
end
-- Periodic random sync STOP

-- Forced full sync command START
SLASH_WOWLOGS_SYNC1 = '/WoWLogsSync'
local function sync(msg, editbox)

	-- I have a gut feeling that this command should not exist
	if(msg == "party") then
		print("Sending "..#syncKeys .." entries to " ..msg)
		local syncNames = ""
		for i = 1, #syncKeys do
			if(db[syncKeys[i]] ~= true and db[syncKeys[i]] ~= false and db[syncKeys[i]] ~= nil) then
				if(string.find(syncKeys[i], "-", string.find(syncKeys[i], "-")+1) ~= nil) then
					WoWLogsSilentPrint("Found faulty key: " ..syncKeys[i])
					db[syncKeys[i]] = nil
					syncKeys[i] = nil
				else
					if((string.len(syncNames) + string.len(syncKeys[i])) < 255) then
						syncNames = syncNames ..syncKeys[i] ..":" 
					else
						C_ChatInfo.SendAddonMessage("WoWLogs", syncNames, "PARTY")
						syncNames = syncKeys[i] ..":"
					end
				end
			end
		end
	end

	-- if(msg ~= "" and msg ~= "party") then
	-- 	print("Sending "..#syncKeys .." entries to " ..msg)
	-- 	for i = 1, #syncKeys do
	-- 		if(db[syncKeys[i]] ~= true and db[syncKeys[i]] ~= false) then
	-- 			C_ChatInfo.SendAddonMessage("WoWLogs", syncKeys[i], "WHISPER", msg)
	-- 		end
	-- 	end
	-- end

	-- if(msg == "") then
	-- 	local name = UnitName("target") .."-" ..string.gsub(GetRealmName()," ", "")
	-- 	print("Sending "..#syncKeys .." entries to " ..name)
	-- 	for i = 1, #syncKeys do
	-- 		if(db[syncKeys[i]] ~= true and db[syncKeys[i]] ~= false) then
	-- 			C_ChatInfo.SendAddonMessage("WoWLogs", syncKeys[i], "WHISPER", name)
	-- 		end
	-- 	end
	-- end


end
SlashCmdList["WOWLOGS_SYNC"] = sync
-- Forced full sync command END

-- ##### CHAT COMMANDS START #####
SLASH_WOWLOGS1 = '/WoWLogs'
local function WoWLogsCommandHandler(msg, editbox)
	if(msg == nil or msg=="" or msg=="help") then
		print("Hai :3")
		print("/wowlogssync [\"party\"/<target>] - Send new playernames to given channel, sending cross-realm only works in parties WARNING: May cause disconnect when sending >3000 users!")
		print("/WoWLogsVisibility [\"always\" / \"never\" / \"alwayslfg\"]")
		print("/WoWLogs \"debug\" - Get debug info in chat")
		print("/WoWLogs \"getpartymembers\" - Fetch all the names of unregistered players, print the average m+ score of those who have logs")
		print("/WoWLogs \"scan\" - Uses the /who command automatically every 15 seconds to fetch users. This is extremely helpful, but MAY open up the who window if you lag")
	end
	if(msg == "debug") then
		if(db["WoWLogs_Debug"] == false) then
			db["WoWLogs_Debug"] = true
		else
			db["WoWLogs_Debug"] = false
		end
	end
	if(msg == "getpartymembers") then
		if(not IsAddOnLoaded("WoWLogsDB")) then
			return;
		end
		local numPlayers = 0
		local mythicPlus = 0
		for i = 1, GetNumGroupMembers() do
			name = "" ..GetRaidRosterInfo(i) 
			if(not string.find(name, "-")) then
				name = name.."-" ..string.gsub(GetRealmName()," ", "")
			end

			if(staticdb[name] == nil) then
				WoWLogsNewEntry(name)
			else
				if(string.sub(staticdb[name],9,11) ~= "000") then
					numPlayers = numPlayers + 1
					mythicPlus = mythicPlus + string.sub(staticdb[name],9,11)
				end
			end
		end
		if(numPlayers ~= 0) then
			mythicPlus = mythicPlus/numPlayers
			print("WoWLogs: Average mythic+ score for " ..numPlayers .." who have logs is " ..mythicPlus)
		end
	end
	if(msg == "scan") then
		-- if(db[WoWLogs_WhoScanner] == nil) then
		-- 	db[WoWLogs_WhoScanner] = "no"
		-- end
		if(db["WoWLogs_WhoScanner"] == "yes" or db["WoWLogs_WhoScanner"] == nil) then
			db["WoWLogs_WhoScanner"] = "no"
			ReloadUI()
		else
			db["WoWLogs_WhoScanner"] = "yes"
			C_Timer.NewTicker(15, WoWLogsWhoScan)
			WoWLogsSilentPrint("Started scanner")
		end
	end
end
SlashCmdList["WOWLOGS"] = WoWLogsCommandHandler
-- ##### CHAT COMMANDS END #####

-- ##### UI VISIBILITY START #####
SLASH_WOWLOGS_VISIBILITY1 = '/WoWLogsVisibility'
local function WoWLogsVisibilityHandler(msg, editbox)
	if(msg == "never") then
		db["WoWLogs_UIState"] = "never"
		WoWLogsTooltip:Hide()
		WoWLogsSilentPrint("Whoosh!")
	end
	if(msg == "always") then
		db["WoWLogs_UIState"] = "always"
		WoWLogsTooltip:Show()
	end
	if(msg == "alwayslfg") then
		db["WoWLogs_UIState"] = "alwayslfg"
		ReloadUI()
	end
	if(msg == "mouseover") then
		db["WoWLogs_UIState"] = "mouseover"
	end
end
SlashCmdList["WOWLOGS_VISIBILITY"] = WoWLogsVisibilityHandler


-- Register all events
for k,v in pairs(events) do
	WoWLogsEventFrame:RegisterEvent(k);
end
WoWLogsEventFrame:SetScript("OnEvent", function (self, event, ...)
	events[event](self, ...);	
end);
GroupFinderFrame:SetScript("OnShow", function ()
	if(db["WoWLogs_UIState"] == "alwayslfg" and not UnitAffectingCombat("player")) then
		WoWLogsTooltip:Show()
	end
end)
GroupFinderFrame:SetScript("OnHide", function ()
	if(db["WoWLogs_UIState"] == "alwayslfg" and not UnitAffectingCombat("player")) then
		WoWLogsTooltip:Hide()
	end
end)
--

print("WoWLogs loading")
	